from .models import NaturalKeyModel, NaturalKeyModelManager, NaturalKeyQuerySet

try:
    from .serializers import NaturalKeySerializer, NaturalKeyModelSerializer
except ImportError:
    NaturalKeySerializer = None
    NaturalKeyModelSerializer = None


__all__ = [
    "NaturalKeyModel",
    "NaturalKeyModelManager",
    "NaturalKeySerializer",
    "NaturalKeyModelSerializer",
    "NaturalKeyQuerySet",
]
